import numpy as np
import matplotlib.colors as colors
import matplotlib.pyplot as plt

from sklearn.datasets import make_blobs
from tools import makeCSVFile

class clusters:
  def __init__(self, centers, std, n_samples, bins, pfparticles, csv_path, event):
    self.centers = centers
    self.std = std
    self.n_samples = n_samples
    self.x, self.y, self.clusters = self.makeClusters()
    self.bins = bins
    self.csv_path = csv_path
    self.pfparticles = pfparticles
    self.event = event
    self.dictionary()
    self.makePixelTruthPlot()
    self.makeDetectorView()

  def makeClusters(self):
    nClusters = len(self.centers)
    X, Y = make_blobs(n_samples=self.n_samples, 
                      centers=self.centers, 
                      cluster_std=self.std,
                      shuffle=True)
    
    x, y, cluster_idxs = [], [], []
    for idx in range(0, nClusters):
        cluster_data = Y == idx
        for x_value, y_value in zip(X[cluster_data, 0], X[cluster_data, 1]):
            x.append(x_value)
            y.append(y_value)
            cluster_idxs.append(idx)

    clusters = []
    for cl in range(0, nClusters):
        cluster_x, cluster_y = [], []
        for x_i in range(0, len(x)):
            if cluster_idxs[x_i] == cl:
                cluster_x.append(x[x_i])
                cluster_y.append(y[x_i])
        clusters.append([cluster_x, cluster_y])
    return x, y, clusters
  
  def getHitsAndClusters(self):
    return self.x, self.y, self.clusters
  
  def dictionary(self):
     dictionary = {'simCluster_energy': [float(nsample) for nsample in self.n_samples],
                   'simCluster_width_x' : [self.std[i] for i in range(len(self.centers))],
                   'simCluster_width_y' : [self.std[i] for i in range(len(self.centers))],
                   'simCluster_width_z' : [0 for _ in range(len(self.centers))],
                   'simCluster_impactPoint_x': [self.centers[i][0] for i in range(len(self.centers))],
                   'simCluster_impactPoint_y': [self.centers[i][1] for i in range(len(self.centers))],
                   'simCluster_impactPoint_z': [0 for _ in range(len(self.centers))],
                   'simCluster_impactPoint_t': [0 for _ in range(len(self.centers))],
                   'simCluster_impactMomentum_x': [0 for _ in range(len(self.centers))],
                   'simCluster_impactMomentum_y': [0 for _ in range(len(self.centers))],
                   'simCluster_impactMomentum_z': self.n_samples,
                   'simCluster_impactMomentum_t': self.n_samples,
                   'simCluster_posBoundary_x': [self.centers[i][0] for i in range(len(self.centers))],
                   'simCluster_posBoundary_y': [self.centers[i][1] for i in range(len(self.centers))],
                   'simCluster_posBoundary_z': [0 for _ in range(len(self.centers))],
                   'simCluster_pdgId': [22 for _ in range(len(self.centers))],
                   'simCluster_layer': [0 for _ in range(len(self.centers))],
                   'simCluster_genParticle': self.pfparticles,
                   'simCluster_event': [self.event for _ in range(len(self.centers))],
                   }
     makeCSVFile(dictionary, self.csv_path)

  def binnedClusters(self):
    nclusters = len(self.clusters)
    bins = self.bins
    binned_clusters_content, binned_clusters_x, binned_clusters_y, binned_clusters_fraction = [], [], [], []

    for clusters in range(0, nclusters):
      content, xedges, yedges = np.histogram2d(self.clusters[clusters][0], self.clusters[clusters][1], bins=bins)
      binned_clusters_content.append(content)
      binned_clusters_x.append([0.5*(xedges[i] + xedges[i-1]) for i in range(1, len(xedges))])
      binned_clusters_y.append([0.5*(yedges[i] + yedges[i-1]) for i in range(1, len(yedges))])

    total_content, max_contribution = [], []
    for x_bin in range(0, len(bins)-1):
      total_content_x = []
      for y_bin in range(0, len(bins)-1):
        max_fraction = 0
        max_fraction_cl = -1
        total_bin_content = 0
        for cluster in range(0, nclusters):
          total_bin_content += binned_clusters_content[cluster][x_bin][y_bin]
          if binned_clusters_content[cluster][x_bin][y_bin] > max_fraction:
            max_fraction = binned_clusters_content[cluster][x_bin][y_bin]
            max_fraction_cl = cluster
        if total_bin_content == 0:
          denominator = float("inf")
        else:
          denominator = total_bin_content
        max_contribution.append([max_fraction_cl, max_fraction / denominator, total_bin_content])
        total_content_x.append(denominator)
      total_content.append(total_content_x)

    for cluster in range(0, nclusters):
      fractions = []
      for x_bin in range(0, len(bins)-1):
        x_fraction = []
        for y_bin in range(0, len(bins)-1):
          fraction = binned_clusters_content[cluster][x_bin][y_bin] / total_content[x_bin][y_bin]
          x_fraction.append(fraction)
        fractions.append(x_fraction)
      binned_clusters_fraction.append(fractions)

    return binned_clusters_content, binned_clusters_x, binned_clusters_y, max_contribution, binned_clusters_fraction
  
  def makePixelTruthPlot(self):
    color = ['white'] + plt.cm.tab20(range(1, len(self.clusters) + 1)).tolist() + plt.cm.tab20b(range(1, len(self.clusters) + 1)).tolist()+ plt.cm.tab20c(range(1, len(self.clusters) + 1)).tolist()
    _, _, _, max_contribution, _ = self.binnedClusters()
    size = len(self.bins) - 1
    min_x = self.bins[0]
    max_x = self.bins[-1]
    min_y = min_x
    max_y = max_x
    cmap = colors.LinearSegmentedColormap.from_list("", color)
    board = np.zeros((size, size))
    fig, ax = plt.subplots(1,1)
    for i in range(size):
      for j in range(size):
        board[i][j] = max_contribution[i + j * size][0]
    ax.imshow(board, cmap=cmap, interpolation='nearest', extent=[min_x, max_x, min_y, max_y])
    ax.invert_yaxis()
    fig.savefig('pixelTruthPlot.png')
    fig.clf()
    plt.close(fig)

  def makeDetectorView(self):
    plt.hist2d(self.x, self.y, bins=self.bins, norm=colors.LogNorm(), cmap='inferno')
    plt.colorbar()
    plt.xlim(self.bins[0], self.bins[-1])
    plt.ylim(self.bins[0], self.bins[-1])
    plt.savefig('detectorView.png')
    plt.clf()
    plt.close()
