#ifndef Hits_h
#define Hits_h

#include "HitsSoA.h"
#include "Tools.h"
#include <cstdint>
#include <vector>
#include <iostream>
// proxy structs for CMSSW hits
/**
 * @class Hit
 * @brief Represents a hit in a detector.
 */
class Hit {

public:
  /**
   * @brief Constructs a Hit object.
   * @param hits The container of hits.
   * @param index The index of the hit in the container.
   */
  //Hit(const HitsSoA &hits, unsigned int &index) : hits_(hits), index(index) {}
  Hit(const float energy, 
      const float x, 
      const float y, 
      const float z, 
      const float time, 
      const uint32_t id) {
    energy_ = energy;
    pos_ = Vector3D(x, y, z);
    time_ = time;
    id_ = id;
  }
  /**
   * @brief Returns the energy of the hit.
   * @return The energy of the hit.
   */
  float energy() const { return energy_; }

  /**
   * @brief Returns the position of the hit.
   * @return The position of the hit as a Vector3D object.
   */
  const Vector3D position() { return pos_; }

  /**
   * @brief Returns the time of the hit.
   * @return The time of the hit.
   */
  float time() const { return time_; }

  /**
   * @brief Returns the ID of the hit.
   * @return The ID of the hit.
   */
  uint32_t id() const { return id_; }

private:
  //const HitsSoA &hits_; // The container of hits.
  Vector3D pos_;
  float energy_, time_;
  uint32_t id_;
  //unsigned int index; // The index of the hit in the container.
};

#endif
