#ifndef SimVertex_h
#define SimVertex_h

#include "Tools.h"
#include <vector>

/**
 * @class SimVertex
 * @brief Represents a simulated vertex in the particle flow truth standalone application.
 * 
 * The SimVertex class stores information about a simulated vertex, including its position,
 * parent index, vertex ID, process type, event ID, and time of flight.
 */
class SimVertex { 
  public:
    SimVertex();
    SimVertex(const Vector3D& v, float tof, int it, unsigned int vId);

    int parentIndex() const { return itrack;}
    bool noParent() const { return itrack == -1; }
    
    void setVertexId(unsigned int n) { vtxId = n; }
    unsigned int vertexId() const { return vtxId; }

    void setProcessType(unsigned int ty) { procType = ty; }
    unsigned int processType() const { return procType; }

    const Vector4D& position() const { return theVertex; }

    void setEventId(unsigned int ev) { event = ev; }
    unsigned int eventId() const { return event; }

    void setTof(float tof) { theVertex.setCoordinates(theVertex.x1, theVertex.x2, theVertex.x3, tof); }
  private:
    int itrack;
    unsigned int vtxId;
    unsigned int procType;

    unsigned int event;
    Vector4D theVertex;
};

typedef std::vector<SimVertex> SimVertexCollection;
#endif
