#include <cmath>

#ifndef Vector3D_h
#define Vector3D_h
// struct for 3D vector
struct Vector3D {
  Vector3D() {
    x = 0.;
    y = 0.;
    z = 0.;
  }
  Vector3D(float xi, float yi, float zi) : x(xi), y(yi), z(zi) {}

  float x, y, z;

  template <typename T> Vector3D operator*(T a) {
    return Vector3D(a * this->x, a * this->y, a * this->z);
  }

  Vector3D operator+(Vector3D v) {
    return Vector3D(this->x + v.x, this->y + v.y, this->z + v.z);
  }

  template <typename T> Vector3D operator/(T a) {
    *this = (*this) * (1 / a);
    return *this;
  }

  Vector3D operator+=(Vector3D v) { 
    *this = *this + v;
    return *this;
  }

  float distance(const Vector3D& v) const {
    return sqrt(std::pow(this->x - v.x, 2) + std::pow(this->y - v.y, 2) +
                std::pow(this->z - v.z, 2));
  }
};
#endif

#ifndef Vector4D_h
#define Vector4D_h
// struct for Vector4D
struct Vector4D {
  Vector4D() {
    x1 = 0.;
    x2 = 0.;
    x3 = 0.;
    y = 0.;
  }
  Vector4D(float x1, float x2, float x3, float y)
      : x1(x1), x2(x2), x3(x3), y(y) {}

  float x1, x2, x3, y;

  void setCoordinates(float x1i, float x2i, float x3i, float yi) {
    x1 = x1i;
    x2 = x2i;
    x3 = x3i;
    y = yi;
  }

  float P() const {
    return sqrt(std::pow(x1, 2) + std::pow(x2, 2) + std::pow(x3, 2));
  }
  float E() { return y; }

  Vector3D position() const {
    return Vector3D(this->x1, this->x2, this->x3);
  }
  float distance(Vector4D& v) const {
    return sqrt(std::pow(this->x1 - v.x1, 2) + std::pow(this->x2 - v.x2, 2) +
                std::pow(this->x3 - v.x3, 2) + std::pow(this->y - v.y, 2));
  }

  Vector4D operator+=(Vector4D v) {
    this->x1 += v.x1;
    this->x2 += v.x2;
    this->x3 += v.x3;
    this->y += v.y;
    return *this;
  }
};
#endif
