#include "interface/DataLoader.h"
#include "interface/HitsSoA.h"
#include "interface/SimCluster.h"
#include "interface/SimClusterMerger.h"
#include "interface/SimHistoryTool.h"
#include "interface/SimVertex.h"

#include "TFile.h"
#include "TInterpreter.h"

#include <cstddef>
#include <iostream>
#include <vector>
#include <memory>

typedef std::vector<std::vector<SimCluster>> SimClusterCollection;
typedef std::vector<std::vector<SimTrack>> SimTrackCollection;
typedef std::vector<std::vector<SimVertex>> SimVertexCollection;
typedef std::vector<HitsSoA> HitsSoACollection;
typedef std::vector<std::vector<PFParticle>> PFParticleCollection;

int main() {

  // generate dictionaries, to be moved
  gInterpreter->GenerateDictionary("vector<vector<float>>", "vector");
  gInterpreter->GenerateDictionary("vector<vector<unsigned int>>", "vector");
  gInterpreter->GenerateDictionary("vector<vector<int>>", "vector");
  gInterpreter->GenerateDictionary("vector<vector<vector<unsigned int>>>", "vector");
  gInterpreter->GenerateDictionary("vector<vector<vector<float>>>", "vector");

  //std::shared_ptr<TFile> input_file =
  //    std::make_shared<TFile>("/work/abrusamolino/simtruth2/CMSSW_14_0_0_pre2/src/"
  //              "Configuration/PFNanoProduction/configs/pfTruthDumper.root", "UPDATE");//,
                //"UPDATE");
  //std::shared_ptr<DataLoader> dataLoader = std::make_shared<DataLoader>(input_file);     // load collections
  
  std::shared_ptr<TFile> _file(TFile::Open("output.root", "RECREATE"));
  std::vector<std::string> input_files_v = {"data/csv/pfParticles.csv", 
                                            "data/csv/hits.csv", 
                                            "data/csv/simClusters.csv", 
                                            "data/csv/simTracks.csv", 
                                            "data/csv/simVertices.csv", 
                                            "data/csv/simHits.csv"};
  std::shared_ptr<DataLoader> dataLoader = std::make_shared<DataLoader>(input_files_v);
  const HitsSoACollection simhits = dataLoader->getSimHitsSoA();
  const HitsSoACollection hits = dataLoader->getHitsSoA(); // get hits
  const SimClusterCollection simClusters =
      dataLoader->getSimClusters(); // get simclusters
  const SimVertexCollection simVertices =
      dataLoader->getSimVertices(); // get simVertices
  const SimTrackCollection simTracks =
      dataLoader->getSimTracks(); // get simtracks
  const PFParticleCollection pfParticles =
      dataLoader->getPFParticles(); // get pfparticles 
  unsigned int nEvents = hits.size();
  SimClusterCollection mergedSimClusters;
  mergedSimClusters.resize(nEvents);

  for (unsigned int i = 0; i < nEvents; ++i) {
    const SimHistoryTool shtools(simTracks[i], simVertices[i]);
    SimClusterMerger scmerger(&hits[i], &shtools);
    
    std::vector<int> masked_scs;
    std::vector<SimCluster> final_merged_scs;
    std::vector<SimCluster> unused_clusters;
    
    //run first iteration of merging
    int n_clusters_from_charged = 0;
    for (const auto& pfParticle : pfParticles[i]) {
      std::vector<SimCluster> sc_first_iteration;
      if (pfParticle.charge() != 0) {
        for (const auto& scIdx : pfParticle.simClusters()) {
            sc_first_iteration.push_back(simClusters[i][scIdx]);
            masked_scs.push_back(scIdx);
            n_clusters_from_charged++;
        }
      }

      //const SimHistoryTool shtools(simTracks[i], simVertices[i]);
      //SimClusterMerger scmerger(&hits[i], &shtools);

      // clusters that are not used in the first iteration
      // merge simclusters
      std::vector<std::vector<size_t>> idxs;
      idxs.resize(sc_first_iteration.size());
      //idxs.resize(simClusters[i].size());
      std::vector<SimCluster> merged_scs_first_iteration =
          scmerger.merge(sc_first_iteration, simTracks[i], unused_clusters, idxs);

      final_merged_scs.insert(final_merged_scs.end(), merged_scs_first_iteration.begin(), merged_scs_first_iteration.end());
    }

    //remove clusters that were used in the first iteration

    std::cout << "==== FIRST ITERATION ====" << std::endl;
    for (auto& sc : final_merged_scs) {
      std::cout << "merged simcluster: " << sc.energy() << std::endl;
    }
    std::cout << "unused clusters: " << unused_clusters.size() << std::endl;
    for (auto& sc : unused_clusters) {
      std::cout << "unused simcluster: " << sc.energy() << std::endl;
    }
    std::cout << "=========================" << std::endl;

    //run second iteration of merging
    std::vector<SimCluster> sc_second_iteration;
    for (unsigned int j = 0; j < simClusters[i].size(); ++j) {
      if (std::find(masked_scs.begin(), masked_scs.end(), j) == masked_scs.end()) {
        sc_second_iteration.push_back(simClusters[i][j]);
      }
    }
   
    sc_second_iteration.insert(sc_second_iteration.end(), unused_clusters.begin(), unused_clusters.end());
    std::vector<std::vector<size_t>> idxs;
    idxs.resize(sc_second_iteration.size());
    std::vector<SimCluster> merged_scs_second_iteration =
        scmerger.merge(sc_second_iteration, idxs);

    std::cout << "==== SECOND ITERATION ====" << std::endl;
    for (auto& sc : merged_scs_second_iteration) {
      std::cout << "merged simcluster: " << sc.energy() << std::endl;
    }
    std::cout << "=========================" << std::endl;

    final_merged_scs.insert(final_merged_scs.end(), merged_scs_second_iteration.begin(), merged_scs_second_iteration.end());
    
    mergedSimClusters[i] = final_merged_scs;

    for (size_t j = 0; j < final_merged_scs.size(); ++j) {
      std::cout << "merged simcluster " << j << " energy: " << final_merged_scs[j].energy() << std::endl;
      std::cout << "merged simcluster " << j << " x: " << final_merged_scs[j].positionAtBoundary().x << std::endl;
      std::cout << "merged simcluster " << j << " y: " << final_merged_scs[j].positionAtBoundary().y << std::endl;
    }
  }

  dataLoader->writeMergedSimClusters(mergedSimClusters);
  _file->Close();
  return 0;
}
