#include "../interface/ROOTLoader.h"
#include <numeric>

void ROOTLoader::fillHitsSoA() {
  std::vector<std::vector<float>> hitsEnergy = fillVector<float>("cell_e");
  std::vector<std::vector<float>> hitsPos_x = fillVector<float>("cell_x");
  std::vector<std::vector<float>> hitsPos_y = fillVector<float>("cell_y");
  std::vector<std::vector<float>> hitsPos_z = fillVector<float>("cell_z");
  //std::vector<std::vector<float>> hitsTime = fillVector<float>("cell_t");
  std::vector<std::vector<uint32_t>> hitsId = fillVector<uint32_t>("cell_id");

  for (unsigned int i = 0; i < nEvents_; ++i) {
    HitsSoA hitsSoA;
    hitsSoA.resize(hitsEnergy[i].size());
    hitsSoA.energy = hitsEnergy[i];
    hitsSoA.x = hitsPos_x[i];
    hitsSoA.y = hitsPos_y[i];
    hitsSoA.z = hitsPos_z[i];
    //hitsSoA.time = hitsTime[i];
    hitsSoA.id = hitsId[i];
    std::vector<float> time;
    time.resize(hitsEnergy[i].size());
    std::iota(time.begin(), time.end(), 0);
    hitsSoA.time = time;
    hitsSoA_.push_back(hitsSoA);
  }
}

void ROOTLoader::fillSimShowers() {
  std::vector<std::vector<float>> simShowerEnergy = fillVector<float>("sim_shower_energy");
  std::vector<std::vector<float>> simShowerPosBoundary_x = fillVector<float>("sim_shower_posBoundary_x");  
  std::vector<std::vector<float>> simShowerPosBoundary_y = fillVector<float>("sim_shower_posBoundary_y");  
  std::vector<std::vector<float>> simShowerPosBoundary_z = fillVector<float>("sim_shower_posBoundary_z");
  std::vector<std::vector<std::vector<uint32_t>>> simShowerHits = fillVector<std::vector<uint32_t>>("sim_shower_hits");
  std::vector<std::vector<std::vector<float>>> simShowerFractions = fillVector<std::vector<float>>("sim_shower_fractions");  
  std::vector<std::vector<int>> simShowerPDGId = fillVector<int>("sim_shower_pdg_id");
  //std::vector<std::vector<int>> simShowerParticle = fillVector<int>("sim_shower_genp");
  
  for (unsigned int i = 0; i < nEvents_; ++i) {
    SimShowerCollection simShowers;
    for (unsigned int j = 0; j < simShowerEnergy[i].size(); ++j) {
      SimShower simShower(
        Vector4D(), // impact momentum
        Vector3D(simShowerPosBoundary_x[i][j], simShowerPosBoundary_y[i][j], simShowerPosBoundary_z[i][j]), // position at boundary
        Vector3D(simShowerPosBoundary_x[i][j], simShowerPosBoundary_y[i][j], simShowerPosBoundary_z[i][j]), // impact point
        simShowerEnergy[i][j], 
        0.f, // cluster width x
        0.f, // cluster width y
        0.f, // cluster width z
        0, // layer
        simShowerHits[i][j],
        simShowerFractions[i][j],
        simShowerPDGId[i][j],
        j
        //simShowerParticle[i][j]
      );
      simShowers.push_back(simShower);
    }
    simShowers_.push_back(simShowers);
  }
}

void ROOTLoader::fillSimTracks() {
  std::vector<std::vector<float>> simTrackMomentum_x = fillVector<float>("sim_track_momentum_x");
  std::vector<std::vector<float>> simTrackMomentum_y = fillVector<float>("sim_track_momentum_y");
  std::vector<std::vector<float>> simTrackMomentum_z = fillVector<float>("sim_track_momentum_z");
  std::vector<std::vector<float>> simTrackMomentum_t = fillVector<float>("sim_track_momentum_e");
  std::vector<std::vector<float>> simTrackTkx = fillVector<float>("sim_track_tkx");
  std::vector<std::vector<float>> simTrackTky = fillVector<float>("sim_track_tky");
  std::vector<std::vector<float>> simTrackTkz = fillVector<float>("sim_track_tkz");
  std::vector<std::vector<float>> simTrackTkmx = fillVector<float>("sim_track_momentum_x");
  std::vector<std::vector<float>> simTrackTkmy = fillVector<float>("sim_track_momentum_y");
  std::vector<std::vector<float>> simTrackTkmz = fillVector<float>("sim_track_momentum_z");
  std::vector<std::vector<float>> simTrackTkmt = fillVector<float>("sim_track_momentum_e");
  std::vector<std::vector<int>> simTrackType = fillVector<int>("sim_track_type");
  //std::vector<std::vector<int>> simTrackVertex = fillVector<int>("sim_track_vertex");
  std::vector<std::vector<int>> simTrackGenp = fillVector<int>("sim_track_genp");
  std::vector<std::vector<int>> simTrackCharge = fillVector<int>("sim_track_charge");

  for (unsigned int i = 0; i < nEvents_; ++i) {
    SimTrackCollection simTracks;
    simTracks.resize(simTrackMomentum_x[i].size());
    for (unsigned int j = 0; j < simTrackMomentum_x[i].size(); ++j) {
      Vector4D momentum(simTrackMomentum_x[i][j], simTrackMomentum_y[i][j],
                        simTrackMomentum_z[i][j], simTrackMomentum_t[i][j]);
      int type = simTrackType[i][j];
      int vertex = j;
      //int vertex = simTrackVertex[i][j];
      int genparticle = simTrackGenp[i][j];
      int charge = simTrackCharge[i][j];
      Vector3D trackPosition(simTrackTkx[i][j], simTrackTky[i][j],
                             simTrackTkz[i][j]);
      Vector4D trackMomentum(simTrackTkmx[i][j], simTrackTkmy[i][j],
                             simTrackTkmz[i][j], simTrackTkmt[i][j]);
      SimTrack simTk(type, charge, momentum, vertex, genparticle, trackPosition,
                     trackMomentum);
      simTracks.push_back(simTk);
    }
    simTracks_.push_back(simTracks);
  }
}

void ROOTLoader::fillSimVertices() {
  std::vector<std::vector<float>> simVertex_x = fillVector<float>("sim_vertex_x");
  std::vector<std::vector<float>> simVertex_y = fillVector<float>("sim_vertex_y");
  std::vector<std::vector<float>> simVertex_z = fillVector<float>("sim_vertex_z");
  std::vector<std::vector<float>> simVertex_t = fillVector<float>("sim_vertex_t");
  //std::vector<std::vector<int>> simVertex_parentIndex = fillVector<int>("sim_vertex_parentIndex");
  //std::vector<std::vector<unsigned int>> simVertex_vertexId = fillVector<unsigned int>("sim_vertex_vertexId");

  for (unsigned int i = 0; i < nEvents_; ++i) {
    SimVertexCollection simVertices;
    simVertices.resize(simVertex_x[i].size());
    for (unsigned int j = 0; j < simVertex_x[i].size(); ++j) {
      Vector3D vertex(simVertex_x[i][j], simVertex_y[i][j], simVertex_z[i][j]);
      SimVertex simVtx(vertex, simVertex_t[i][j], 
                       //simVertex_parentIndex[i][j],
                       j,
                       //simVertex_vertexId[i][j]);
                       j);
      simVertices.push_back(simVtx);
    }
    simVertices_.push_back(simVertices);
  }  
}

void ROOTLoader::saveMergedSimShowers(std::vector<SimShowerCollection>& merged_showers) {

  file_->cd();
  TTree* tree = new TTree("mergedSimClusters", "mergedSimClusters");
  setBranches(tree);
  for (unsigned int i = 0; i < nEvents_; ++i) {
    //tree_->GetEntry(i);
    clear();
    unsigned int nMergedSimShowers = merged_showers[i].size();
    std::cout << "Event: " << i << std::endl;
    std::cout << "MergedSimShowers size: " << nMergedSimShowers << std::endl;
    //hits[i].resize(nMergedSimShowers);
    //fractions[i].resize(nMergedSimShowers);
    hits.resize(nMergedSimShowers);
    fractions.resize(nMergedSimShowers);
    tree_->GetEntry(i);
    for (unsigned int j = 0; j < nMergedSimShowers; ++j) {
      posBoundary_x.push_back(merged_showers[i][j].positionAtBoundary().x);
      posBoundary_y.push_back(merged_showers[i][j].positionAtBoundary().y);
      posBoundary_z.push_back(merged_showers[i][j].positionAtBoundary().z);
      impactPoint_x.push_back(merged_showers[i][j].impactPoint().x);
      impactPoint_y.push_back(merged_showers[i][j].impactPoint().y);
      impactPoint_z.push_back(merged_showers[i][j].impactPoint().z);
      impactMomentum_x.push_back(merged_showers[i][j].impactMomentum().x1);
      impactMomentum_y.push_back(merged_showers[i][j].impactMomentum().x2);
      impactMomentum_z.push_back(merged_showers[i][j].impactMomentum().x3);
      impactMomentum_t.push_back(merged_showers[i][j].impactMomentum().y);
      energy.push_back(merged_showers[i][j].energy());
      layer.push_back(merged_showers[i][j].getLayer());
      pdgId.push_back(merged_showers[i][j].pdgId());

      event.push_back(i);
      auto haf = merged_showers[i][j].hits_and_fractions();
      //hits[i][j].resize(haf.size());
      //fractions[i][j].resize(haf.size());
      for (unsigned int k = 0; k < haf.size(); ++k) {
        //hits[i][j] = haf[k].first;
        hits[j].push_back(haf[k].first);
        fractions[j].push_back(haf[k].second);
      }
    }
    tree->Fill();
    clear();
  }
  tree->Write("", TObject::kOverwrite);
}
